from __future__ import unicode_literals

import sys
import os
import logging
from nose.plugins import Plugin

log = logging.getLogger('nose.plugins.nose_exclude')

if sys.version_info > (3,):
    get_method_class = lambda x: x.__self__.__class__
else:
    get_method_class = lambda x: x.im_class


class NoseExclude(Plugin):

    def options(self, parser, env=os.environ):
        """Define the command line options for the plugin."""
        super(NoseExclude, self).options(parser, env)
        env_dirs = []
        env_tests = []

        if 'NOSE_EXCLUDE_DIRS' in env:
            exclude_dirs = env.get('NOSE_EXCLUDE_DIRS', '')
            env_dirs.extend(exclude_dirs.split(';'))

        parser.add_option(
            str("--exclude-dir"), action="append",
            dest="exclude_dirs",
            default=env_dirs,
            help="Directory to exclude from test discovery. \
                Path can be relative to current working directory \
                or an absolute path. May be specified multiple \
                times. [NOSE_EXCLUDE_DIRS]")

        parser.add_option(
            str("--exclude-dir-file"), type="string",
            dest="exclude_dir_file",
            default=env.get('NOSE_EXCLUDE_DIRS_FILE', False),
            help="A file containing a list of directories to exclude \
                from test discovery. Paths can be relative to current \
                working directory or an absolute path. \
                [NOSE_EXCLUDE_DIRS_FILE]")

        parser.add_option(
            str("--exclude-test"), action="append",
            dest="exclude_tests",
            default=env_tests,
            help="Fully qualified name of test method or class to exclude \
            from test discovery.")

        parser.add_option(
            str("--exclude-test-file"), type="string",
            dest="exclude_test_file",
            default=False,
            help="A file containing a list of fully qualified names of \
                test methods or classes to exclude from test discovery.")

    def _force_to_abspath(self, pathname, root):
        if os.path.isabs(pathname):
            abspath = pathname
        else:
            abspath = os.path.abspath(os.path.join(root, pathname))

        if os.path.exists(abspath):
            return abspath
        else:
            log.warning('The following path was not found: %s' % pathname)

    def _load_from_file(self, filename):
        with open(filename, 'r') as infile:
            new_list = [l.strip() for l in infile.readlines() if l.strip()
                        and not l.startswith('#')]
        return new_list

    def configure(self, options, conf):
        """Configure plugin based on command line options"""
        super(NoseExclude, self).configure(options, conf)

        self.exclude_dirs = {}
        self.exclude_tests = options.exclude_tests[:]

        # preload directories from file
        if options.exclude_dir_file:
            if not options.exclude_dirs:
                options.exclude_dirs = []

            new_dirs = self._load_from_file(options.exclude_dir_file)
            options.exclude_dirs.extend(new_dirs)

        if options.exclude_test_file:
            exc_tests = self._load_from_file(options.exclude_test_file)
            self.exclude_tests.extend(exc_tests)

        if not options.exclude_dirs and not self.exclude_tests:
            self.enabled = False
            return

        self.enabled = True
        if conf and conf.workingDir:
            # Use nose's working directory
            root = conf.workingDir
        else:
            root = os.getcwd()

        log.debug('cwd: %s' % root)

        # Normalize excluded directory names for lookup
        for exclude_param in options.exclude_dirs:
            # when using setup.cfg, you can specify only one 'exclude-dir'
            # separated by some character (new line is good enough)
            for d in exclude_param.split('\n'):
                d = d.strip()
                abs_d = self._force_to_abspath(d, root)
                if abs_d:
                    self.exclude_dirs[abs_d] = True

        exclude_str = "excluding dirs: %s" % ",".join(list(self.exclude_dirs.keys()))
        log.debug(exclude_str)

    def wantDirectory(self, dirname):
        """Check if directory is eligible for test discovery"""
        # In case of symbolic paths
        dirname = os.path.realpath(dirname)

        if dirname in self.exclude_dirs:
            log.debug("excluded: %s" % dirname)
            return False
        else:
            return None

    def wantModule(self, module):
        """Filter out tests based on: <module path>.<module name>"""
        if module.__name__ in self.exclude_tests:
            return False
        else:
            return None

    def wantFunction(self, fun):
        """Filter out tests based on: <module path>.<func name>"""
        fqn = '%s.%s' % (fun.__module__, fun.__name__)
        if fqn in self.exclude_tests:
            return False
        else:
            return None

    def wantMethod(self, meth):
        """Filter out tests based on <module path>.<class>.<method name>"""
        try:
            cls = get_method_class(meth)
        except AttributeError:
            return False

        fqn = '%s.%s.%s' % (cls.__module__, cls.__name__, meth.__name__)
        if fqn in self.exclude_tests:
            return False
        else:
            return None

    def wantClass(self, cls):
        """Filter out the class based on <module path>.<class name>"""
        fqn = '%s.%s' % (cls.__module__, cls.__name__)
        if fqn in self.exclude_tests:
            return False
        else:
            return None



import os
import unittest
from nose.plugins import PluginTester
from nose_exclude import NoseExclude


class TestNoseExcludeDirs_Relative_Args(PluginTester, unittest.TestCase):
    """Test nose-exclude directories using relative paths passed
    on the commandline via --exclude-dir
    """

    activate = "--exclude-dir=test_dirs/build"
    args = ['--exclude-dir=test_dirs/test_not_me']
    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs')

    def test_proper_dirs_omitted(self):
        assert "FAILED" not in self.output


class TestNoseExcludeDirs_Absolute_Args(PluginTester, unittest.TestCase):
    """Test nose-exclude directories using absolute paths passed
    on the commandline via --exclude-dir
    """

    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs')

    def __init__(self, *args, **kwargs):
        self.activate = "--exclude-dir=%s" % \
                os.path.join(self.suitepath, 'build')
        arg_path = os.path.join(self.suitepath, 'test_not_me')
        self.args = ['--exclude-dir=%s' % arg_path]
        super(TestNoseExcludeDirs_Absolute_Args, self).__init__(*args, **kwargs)

    def test_proper_dirs_omitted(self):
        assert "FAILED" not in self.output


class TestNoseExcludeDirs_Relative_Args_File(PluginTester, unittest.TestCase):
    """Test nose-exclude directories using relative paths passed
    by file using --exclude-dir-file
    """

    activate = "--exclude-dir-file=test_dirs/exclude_dirs.txt"
    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs')

    def test_proper_dirs_omitted(self):
        assert "FAILED" not in self.output


class TestNoseExcludeDirs_Relative_Args_Mixed(PluginTester, unittest.TestCase):
    """Test nose-exclude directories using paths passed
    by file and commandline
    """

    activate = "--exclude-dir-file=test_dirs/exclude_dirs2.txt"
    args = ["--exclude-dir=test_dirs/test_not_me"]
    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs')

    def test_proper_dirs_omitted(self):
        assert "FAILED" not in self.output


class TestNoseExcludeEnvVariables(PluginTester, unittest.TestCase):
    """Test nose-exclude's use of environment variables"""

    #args = ['--exclude-dir=test_dirs/test_not_me']
    activate = "-v"
    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs')

    env = {'NOSE_EXCLUDE_DIRS': 'test_dirs/build;test_dirs/test_not_me'}

    def test_proper_dirs_omitted(self):
        assert "FAILED" not in self.output


class TestNoseExcludeDirsEnvFile(PluginTester, unittest.TestCase):
    """Test nose-exclude directories using relative paths passed
    by file specified by environment variable
    """

    activate = "-v"
    plugins = [NoseExclude()]
    env = {'NOSE_EXCLUDE_DIRS_FILE': 'test_dirs/exclude_dirs.txt'}
    suitepath = os.path.join(os.getcwd(), 'test_dirs')

    def test_proper_dirs_omitted(self):
        assert "FAILED" not in self.output


class TestNoseExcludeDirs_Arg_Does_Not_Exist(PluginTester, unittest.TestCase):
    """Test nose-exclude directories for a directory that doesn't exist.
    """

    activate = "--exclude-dir=test_dirs/build"
    args = ["--exclude-dir=test_dirs/test_not_me \n --exclude-dir=test_dirs/test_i_dont_exist"]
    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs')

    def test_proper_dirs_omitted(self):
        assert "FAILED" not in self.output


class TestNoseExcludeDirsNoseWorkingDir(PluginTester, unittest.TestCase):
    """Test nose-exclude directories with Nose's working directory."""

    activate = "--exclude-dir=test_not_me"
    args = ["--where=test_dirs"]
    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs')

    def tearDown(self):
        # Nose changes cwd to config.workingDir, need to reset it
        import os
        os.chdir(os.path.join(os.getcwd(), os.path.pardir))

    def test_proper_dirs_omitted(self):
        assert "FAILED" not in self.output


class TestNoseExcludeTest(PluginTester, unittest.TestCase):
    """Test nose-exclude a single test"""

    activate = "--exclude-test=test_dirs.unittest.tests.UnitTests.test_a"
    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs/unittest')

    def test_test_excluded(self):
        assert 'Ran 2 tests' in self.output


class TestNoseExcludeTestNegative(PluginTester, unittest.TestCase):
    """Test nose-exclude a test that does not exist"""

    activate = "--exclude-test=test_dirs.unittest.tests.UnitTests.does_not_exist"
    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs/unittest')

    def test_test_excluded_negative(self):
        assert 'Ran 3 tests' in self.output


class TestNoseExcludeMultipleTest(PluginTester, unittest.TestCase):
    """Test nose-exclude multiple tests"""

    activate = "--exclude-test=test_dirs.unittest.tests.UnitTests.test_a"
    args = [
        "--exclude-test=test_dirs.unittest.tests.UnitTests.test_b",
        "--exclude-test=test_dirs.unittest.tests.test_c"
    ]
    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs/unittest')

    def test_tests_excluded(self):
        assert 'Ran 0 tests' in self.output


class TestNoseExcludeTestViaFile(PluginTester, unittest.TestCase):
    """Test nose-exclude tests with a file"""

    activate = "--exclude-test-file=test_dirs/exclude_tests.txt"
    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs/unittest')

    def test_tests_excluded(self):
        assert 'Ran 1 test' in self.output


class TestNoseExcludeDirAndTests(PluginTester, unittest.TestCase):
    """Test nose-exclude tests by specifying dirs and tests"""

    activate = "--exclude-test=test_dirs.unittest.tests.UnitTests.test_a"
    args = [
        "--exclude-dir=test_dirs/build",
        "--exclude-dir=test_dirs/build2",
        "--exclude-dir=test_dirs/fish",
        "--exclude-dir=test_dirs/test_not_me",
        "--exclude-dir=test_dirs/test_yes",
        "--exclude-dir=test_dirs/test_yes2",
    ]
    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs')

    def test_tests_excluded(self):
        assert 'Ran 2 tests' in self.output


class TestNoseExcludeTestClass(PluginTester, unittest.TestCase):
    """Test nose-exclude tests by class"""

    activate = "--exclude-test=test_dirs.unittest.tests.UnitTests"

    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs/unittest')

    def test_tests_excluded(self):
        assert 'Ran 1 test' in self.output


class TestNoseExcludeTestFunction(PluginTester, unittest.TestCase):
    """Test nose-exclude tests by function"""

    activate = "--exclude-test=test_dirs.unittest.tests.test_c"

    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs/unittest')

    def test_tests_excluded(self):
        assert 'Ran 2 tests' in self.output


class TestNoseExcludeTestModule(PluginTester, unittest.TestCase):
    """Test nose-exclude tests by module"""

    activate = "--exclude-test=test_dirs.unittest.test"

    plugins = [NoseExclude()]
    suitepath = os.path.join(os.getcwd(), 'test_dirs/unittest')

    def test_tests_excluded(self):
        assert 'Ran 3 tests' in self.output

if __name__ == '__main__':
    unittest.main()



# Multiprocessing handling to avoid atexit test runner failure
try:
    import multiprocessing
except ImportError:
    pass
import os
from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read().strip()

VERSION = '0.4.1'

setup(
    name="nose-exclude",
    version=VERSION,
    author="Kurt Grandis",
    author_email="kgrandis@gmail.com",
    description="Exclude specific directories from nosetests runs.",
    long_description=read('README.rst'),
    license='GNU LGPL',
    url="https://github.com/kgrandis/nose-exclude",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        ("License :: OSI Approved :: GNU Library or Lesser General "
         "Public License (LGPL)"),
        "Topic :: Software Development :: Testing",
        "Programming Language :: Python",
        ],

    py_modules=['nose_exclude'],
    zip_safe=False,

    entry_points={
        'nose.plugins': ['nose_exclude = nose_exclude:NoseExclude']
        },
    install_requires=['nose'],
    test_suite='tests',
)
